with Ada.Text_IO;

with Names;
with HW_Simulation;


package body UnitMgr.Branch_Control is

   use Names;

   procedure Init_Transition (
      UnId : Unit_ID_T;
      BrId : Branch_ID_T ) is
   begin
      Units(UnId).Br(BrId).Target_State := None;
      Units(UnId).Br(BrId).Step := 0;
      Units(UnId).Br(BrId).Timer := 0;
   end Init_Transition;

   procedure Possible_State_Transition_Step (
      UnId : Unit_ID_T;
      BrId : Branch_ID_T )
   is
      Prev_Step : Natural;
      Ret_Value_from_Call : Boolean;
   begin
      if Units(UnId).Br(BrId).Target_State /= None then
         -- Transition to new operational state is ongoing
         Prev_Step := Units(UnId).Br(BrId).Step;
         Ada.Text_IO.Put_Line ("br="&UnId'Img & " " & BrId'Img
            & " stp=" & Units(UnId).Br(BrId).Step'Img
            & " st=" & Units(UnId).Br(BrId).State'Img
            & " tst=" & Units(UnId).Br(BrId).Target_State'Img);
         if Units(UnId).Br(BrId).Target_State = Off
            or Units(UnId).Br(BrId).State = Off
         then
            if Units(UnId).Br(BrId).Step = 0 then
               HW_Simulation.Receive_HW_Signals (UnId, BrId,
                  Units(UnId).Br(BrId).Target_State = Off);
               Units(UnId).Br(BrId).Step := 1;
            elsif Units(UnId).Br(BrId).Target_State = Off then
               if Units(UnId).Br(BrId).Timer >= 3 then
                  -- Unconditionally declare the unit to be in Off state
                  -- after 3 cycles, no check of the real sending or effect
                  -- of the H/W signals
                  Units(UnId).Br(BrId).State := Off;
               end if;
            else
               Ret_Value_from_Call := HW_Simulation.Powered (UnId, BrId);
               if Ret_Value_from_Call then
                  Units(UnId).Br(BrId).State := Least_Non_Off_State(UnId);
                  if Units(UnId).Br(BrId).State /=
                     Units(UnId).Br(BrId).Target_State
                  then
                     Units(UnId).Br(BrId).Step := 2;
                  end if;
               end if;
            end if;
         elsif UnId = GPS or UnId = PLI then
            if Units(UnId).Br(BrId).Step = 0
               or Units(UnId).Br(BrId).Step = 2
            then 
               if Units(UnId).Br(BrId).Step = 0
                  and Units(UnId).Br(BrId).State = Least_Non_Off_State(UnId)
               then
                  Units(UnId).Br(BrId).Step := 2;
               end if;
               HW_Simulation.Receive_TC (UnId, BrId,
                  Target_State_TC(Units(UnId).Br(BrId).Target_State));
               Units(UnId).Br(BrId).Step := Units(UnId).Br(BrId).Step + 1;
            else
               Ret_Value_from_Call := HW_Simulation.Event_Generated (
                  UnId, BrId,
                  Target_State_Event(Units(UnId).Br(BrId).Target_State));
               if Ret_Value_from_Call then
                  Units(UnId).Br(BrId).State := Units(UnId).Br(BrId).Target_State;
               end if;
            end if;
         end if;
         if Units(UnId).Br(BrId).State = Units(UnId).Br(BrId).Target_State then
            -- Transition to new operational state is over
            Init_Transition (UnId, BrId);
         elsif Prev_Step < Units(UnId).Br(BrId).Step then
            -- One step handled
            Units(UnId).Br(BrId).Timer := 0;
         else
            Units(UnId).Br(BrId).Timer := Units(UnId).Br(BrId).Timer + 1;
            if Units(UnId).Br(BrId).Timer > Units(UnId).Br(BrId).Timeout_Limit then
               Init_Transition (UnId, BrId);
               Units(UnId).Br(BrId).Error := Timeout;
               Ada.Text_IO.Put_Line ("utimeo br="&UnId'Img & " " & BrId'Img);
            end if;
         end if;
      end if;
   end Possible_State_Transition_Step;

end UnitMgr.Branch_Control;
